"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const mobile_devices_controller_1 = require("mobile-devices-controller");
class DeviceController {
    static startDevice(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const devices = (yield mobile_devices_controller_1.DeviceManager.getAllDevices(args.appiumCaps.platformName.toLowerCase(), args.appiumCaps.deviceName));
            if (!devices || devices === null || devices.size) {
                console.log("Available devices:\n", devices);
                throw new Error(`No such device ${args.appiumCaps.deviceName}!!!\n Check availabe emulators!!!`);
            }
            let device;
            // Should find new device
            if (!args.reuseDevice) {
                device = DeviceController.getDevicesByStatus(devices, mobile_devices_controller_1.Status.SHUTDOWN);
            }
            // If there is no  shutdown device
            if (!device || device === null) {
                device = DeviceController.getDevicesByStatus(devices, mobile_devices_controller_1.Status.BOOTED);
            }
            // If there is no booted device
            if (!device || device === null) {
                device = DeviceController.getDevicesByStatus(devices, mobile_devices_controller_1.Status.FREE);
            }
            // In case reuse device is true but there weren't any booted devices. We need to fall back and boot new one.
            if (!device || device === null && args.reuseDevice) {
                device = DeviceController.getDevicesByStatus(devices, mobile_devices_controller_1.Status.SHUTDOWN);
            }
            if (device.status === mobile_devices_controller_1.Status.SHUTDOWN) {
                yield mobile_devices_controller_1.DeviceManager.startDevice(device);
                console.log("Started device: ", device);
            }
            else {
                console.log("Device is alredy started", device);
                if (!args.reuseDevice) {
                    DeviceController.kill(device);
                    yield mobile_devices_controller_1.DeviceManager.startDevice(device);
                }
            }
            DeviceController._emulators.set(args.runType, device);
            return device;
        });
    }
    static stop(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (DeviceController._emulators.has(args.runType) && !args.reuseDevice) {
                const device = DeviceController._emulators.get(args.runType);
                yield mobile_devices_controller_1.DeviceManager.kill(device);
            }
        });
    }
    static kill(device) {
        return __awaiter(this, void 0, void 0, function* () {
            yield mobile_devices_controller_1.DeviceManager.kill(device);
        });
    }
    static device(runType) {
        return DeviceController._emulators.get(runType);
    }
    static getDevicesByStatus(devices, status) {
        let device;
        const shutdownDeivces = devices.filter(dev => {
            return dev.status === status;
        });
        if (shutdownDeivces && shutdownDeivces !== null && shutdownDeivces.length > 0) {
            device = shutdownDeivces[0];
        }
        return device;
    }
}
DeviceController._emulators = new Map();
exports.DeviceController = DeviceController;
//# sourceMappingURL=device-controller.js.map